package cloudduggu.com;

import org.apache.spark.SparkConf;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Row;
import org.apache.spark.sql.SparkSession;
import org.apache.spark.streaming.Durations;
import org.apache.spark.streaming.api.java.JavaDStream;
import org.apache.spark.streaming.api.java.JavaStreamingContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

/**
 * Spark streaming code to calculate nifty bank stock data.
 *
 * @author  Sarvesh Kumar (CloudDuggu.com)
 * @version 1.0
 * @since   2020-08-05
 */

public class NiftyBankStockStreaming {

    static final Logger logger = LoggerFactory.getLogger(NiftyBankStockStreaming.class);

    static int count = 0;
    static int error = 0;

    public static void main(String[] args) throws Exception {

        // Create a local StreamingContext with batch interval of 10 second
        SparkConf sparkConf = new SparkConf().setAppName("CluddugguSparkStreaming");
        JavaStreamingContext ssc = new JavaStreamingContext(sparkConf, Durations.seconds(10));

        JavaDStream<String> fileStream =ssc.textFileStream(args[0]);
        JavaDStream<String> lines = fileStream.flatMap(x -> Arrays.asList(x.split("\n")).iterator());


        lines.foreachRDD(rdd ->{
            if(!rdd.isEmpty()) {
                try {
                    // Get the singleton instance of SparkSession
                    SparkSession spark = SparkSession.builder().config(sparkConf).getOrCreate();

                    // Convert RDD[String] to RDD[NiftyBank]
                    JavaRDD<NiftyBank> rowRDD = rdd.map(line -> {
                        String col[] = line.split(",");
                        NiftyBank niftyBank = new NiftyBank();
                        niftyBank.setSymbol(col[0]);
                        niftyBank.setOpen(Float.parseFloat(col[1]));
                        niftyBank.setHigh(Float.parseFloat(col[2]));
                        niftyBank.setLow(Float.parseFloat(col[3]));
                        niftyBank.setLtp(Float.parseFloat(col[4]));
                        niftyBank.setClose(Float.parseFloat(col[5]));
                        niftyBank.set_52wh(Float.parseFloat(col[6]));
                        niftyBank.set_52wl(Float.parseFloat(col[7]));
                        niftyBank.setVolume(Double.parseDouble(col[8]));
                        niftyBank.setValue(Double.parseDouble(col[9]));
                        niftyBank.setNoOfTrades(Long.parseLong(col[10]));
                        return niftyBank;
                    });

                    // Create RDD[NiftyBank] to Dataset[Row]
                    Dataset<Row> wordsDataFrame = spark.createDataFrame(rowRDD, NiftyBank.class);

                    // Creates a temporary view using the DataFrame
                    wordsDataFrame.createOrReplaceTempView("bank");

                    // Do word count on table using SQL and print it
                    Dataset<Row> teenagerDS = spark.sql("select symbol, avg(ltp) from bank group by symbol");

//                    teenagerDS.show();

                    Map dataMap = new HashMap();
                    teenagerDS.collectAsList().stream()
                        .forEach(data -> dataMap.put(data.get(0),data.get(1))
                    );

//                    KOTAKBANK,HDFCBANK,INDUSINDBK,AXISBANK,ICICIBANK,

                    String line = "KOTAKBANK,HDFCBANK,INDUSINDBK,AXISBANK,ICICIBANK" +"\n"+
                    dataMap.get("KOTAKBANK")+","+dataMap.get("HDFCBANK")+","+dataMap.get("INDUSINDBK")+","+dataMap.get("AXISBANK")+","+dataMap.get("ICICIBANK");

                    logger.info("==============================================================");
                    logger.info("["+count+"] Result: "+line);
                    logger.info("==============================================================");

                    // Upload Result Output
                    String encodeURL= "http://"+args[1]+":"+args[2]+"/upload?result=" + URLEncoder.encode( line, "UTF-8" );
                    URL resUrl = new URL(encodeURL);
                    HttpURLConnection resConn = (HttpURLConnection) resUrl.openConnection();
                    resConn.setRequestMethod("POST");
                    int responseCode = resConn.getResponseCode();

                    logger.info("==============================================================");
                    logger.info("["+count+"] Upload Success... : "+responseCode);
                    logger.info("==============================================================");

                    count++;
                    error = 0;

                    if(count == 10) {
                        logger.info("==============================================================");
                        logger.info("["+count+"] Stop Processing... : ");
                        logger.info("==============================================================");
                        System.exit(0);
                    }
                } catch (Exception ex) {
                    logger.info("==============================================================");
                    ex.printStackTrace();
                    logger.info("==============================================================");
                }
            } else {
                error++;
                logger.info("["+error+"] Data Not Found... : ");
                if(error == 10) {
                    logger.info("==============================================================");
                    logger.info("["+count+"] Stop Processing... : ");
                    logger.info("==============================================================");
                    System.exit(0);
                }
            }
        });

        ssc.start();
        ssc.awaitTermination();
    }
}
